#include <stdio.h>
#include <string.h>
#include <DataIO.h>
#include "zfont.h"

status_t fonts_status_to_stdout(const char *filename)
{
	FILE *f;
	int err;
	uint32 fonts_status_size, status_count;
	fonts_status_t fonts_status;
	
	f = fopen(filename, "r");
	if (!f) return errno;
	
	if (fseek(f, 0x03c8, SEEK_SET) == -1) {
		err = errno;
		fclose(f);
		return err;
	}
	
	if (fread(&fonts_status_size, 1, sizeof(fonts_status_size), f) < sizeof(fonts_status_size)) {
		fclose(f);
		return B_ERROR;
	}
	
	status_count = fonts_status_size / sizeof(fonts_status_t);
	for(uint32 i = 0; i < status_count; i++) {
		if (fread(&fonts_status, 1, sizeof(fonts_status), f) < sizeof(fonts_status)) {
			fclose(f);
			return B_ERROR;
		}
		printf(":%4.4x %4.4x %s\n", fonts_status.begin_code, fonts_status.end_code, fonts_status.font_name);
	}
	
	fclose(f);
	
	return B_OK;
}

status_t stdin_to_fonts_status(const char *filename)
{
	char pre_data[0x3c8], suf_data[72];
	int err = B_OK;
	FILE *f;
	uint32 fonts_status_size;
	fonts_status_t fonts_status;
	unsigned int begin_code, end_code;
	char buff[66];
	int c, line = 0;
	BMallocIO mallocIO;
	
	f = fopen(filename, "r");
	if (!f) return errno;
	
	if (fread(pre_data, 1, sizeof(pre_data), f) < sizeof(pre_data)) {
		fclose(f);
		return B_ERROR;
	}
	mallocIO.Write(pre_data, sizeof(pre_data));
	
	if (fread(&fonts_status_size, 1, sizeof(fonts_status_size), f) < sizeof(fonts_status_size)) {
		fclose(f);
		return B_ERROR;
	}
	mallocIO.Write(&fonts_status_size, sizeof(fonts_status_size));
	
	if (fseek(f, fonts_status_size, SEEK_CUR) == -1) {
		err = errno;
		fclose(f);
		return err;
	}
	
	if (fread(&suf_data, 1, sizeof(suf_data), f) < sizeof(suf_data)) {
		fclose(f);
		return B_ERROR;
	}
	
	while(!feof(stdin)) {
		line++;
		memset(&buff, 0, sizeof(buff));
		memset(&fonts_status, 0, sizeof(fonts_status));
		fonts_status.size = sizeof(fonts_status_t);
		
		c = fgetc(stdin);
		if (feof(stdin)) break;
		if (c != ':') { err = B_ERROR; break; }
		if (fscanf(stdin, "%x ", &begin_code) != 1) { err = B_ERROR; break; }
		if (fscanf(stdin, "%x ", &end_code) != 1) { err = B_ERROR; break; }
		fgets(buff, sizeof(buff), stdin);
		
		fonts_status.begin_code = begin_code;
		fonts_status.end_code = end_code;
		for(uint32 i = 0; i < sizeof(buff); i++) {
			if (buff[i] == '\n') buff[i] = 0;
		}
		memcpy(fonts_status.font_name, buff, sizeof(fonts_status.font_name));
		mallocIO.Write(&fonts_status, sizeof(fonts_status));
	}
	
	if (err != B_OK) fprintf(stderr, "Invalid data at %d.\nStop.\n", line);
	
	fclose(f);
	
	mallocIO.Write(suf_data, sizeof(suf_data));
	
	fonts_status_size = (line - 1) * sizeof(fonts_status_t);
	mallocIO.WriteAt(sizeof(pre_data), &fonts_status_size, sizeof(fonts_status_size));
	
	fwrite(mallocIO.Buffer(), 1, mallocIO.BufferLength(), stdout);
	
	return B_OK;
}

void show_usage()
{
	fprintf(stderr, "Usage: zfont [options] [path to fonts_status (default: /boot/home/config/settings/fonts_status)]\n");
	fprintf(stderr, "Options:\n");
	fprintf(stderr, "  --import    Create fonts_status from STDIN.\n");
	fprintf(stderr, "  --export    Show fonts_status.\n");
}

int main(int argc, char *argv[])
{
	char *fonts_status_path;
	status_t e = B_OK;
	
	if (argc < 2) {
		show_usage();
		return 0;
	}
	
	if (argc == 2) {
		fonts_status_path = path_to_fonts_status;
	} else {
		fonts_status_path = argv[2];
	}
	
	if (strcmp(argv[1], "--export") == 0) {
		fprintf(stderr, "Reading from '%s'\n", fonts_status_path);
		e = fonts_status_to_stdout(fonts_status_path);
	} else if (strcmp(argv[1], "--import") == 0) {
		e = stdin_to_fonts_status(fonts_status_path);
	} else {
		show_usage();
		return 0;
	}
	
	if (e != B_OK) {
		fprintf(stderr, "%s\n", strerror(e));
		return -1;
	}
	
	return 0;
}
